@@ -292,7 +292,15 @@ enum class TSG_Array_Growth
 	SG_ARRAY_GROWTH_0 = 0,
 	SG_ARRAY_GROWTH_1,
 	SG_ARRAY_GROWTH_2,
-	SG_ARRAY_GROWTH_3
+	SG_ARRAY_GROWTH_3,
+	SG_ARRAY_GROWTH_FIX_8,
+	SG_ARRAY_GROWTH_FIX_16,
+	SG_ARRAY_GROWTH_FIX_32,
+	SG_ARRAY_GROWTH_FIX_64,
+	SG_ARRAY_GROWTH_FIX_128,
+	SG_ARRAY_GROWTH_FIX_256,
+	SG_ARRAY_GROWTH_FIX_512,
+	SG_ARRAY_GROWTH_FIX_1024
 };
 
 //---------------------------------------------------------
@@ -772,14 +780,6 @@ SAGA_API_DLL_EXPORT CSG_Strings		SG_String_Tokenize				(const CSG_String &String
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT bool			SG_is_Character_Numeric			(int Character);
 
-SAGA_API_DLL_EXPORT int				SG_Printf						(const CSG_String &String);
-SAGA_API_DLL_EXPORT int				SG_Printf						(const  char   *Format, ...);
-SAGA_API_DLL_EXPORT int				SG_Printf						(const wchar_t *Format, ...);
-
-SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const CSG_String &String);
-SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const  char   *Format, ...);
-SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const wchar_t *Format, ...);
-
 SAGA_API_DLL_EXPORT CSG_String		SG_Get_CurrentTimeStr			(bool bWithDate = true);
 
 SAGA_API_DLL_EXPORT double			SG_Degree_To_Double				(const CSG_String &String);
@@ -922,47 +922,45 @@ private:
 class CSG_Stack
 {
 public:
-	CSG_Stack(size_t RecordSize) : m_Size(0), m_Buffer(0), m_RecordSize(RecordSize), m_Stack(NULL)	{}
-	virtual ~CSG_Stack(void)						{	Destroy();				}
-
-	size_t					Get_RecordSize	(void)	{	return( m_RecordSize );	}
-	size_t					Get_Size		(void)	{	return( m_Size );		}
-	void					Clear			(void)	{	m_Size	= 0;			}
-
-	void					Destroy			(void)
+	CSG_Stack(size_t RecordSize)
 	{
-		if( m_Stack )
-		{
-			SG_Free(m_Stack);
-		}
+		m_Stack.Create(RecordSize, 1, TSG_Array_Growth::SG_ARRAY_GROWTH_FIX_256);
+	}
 
-		m_Size		= 0;
-		m_Buffer	= 0;
-		m_Stack		= NULL;
+	virtual ~CSG_Stack(void) {}
+
+	bool					Destroy			(void)       { return( m_Stack.Set_Array(1) ); }
+
+	size_t					Get_RecordSize	(void) const { return( m_Stack.Get_Value_Size() ); }
+	size_t					Get_Size		(void) const { return( m_Stack.Get_uSize() - 1 ); }
+
+	bool					Clear			(bool bFreeMemory = false)
+	{
+		return( m_Stack.Set_Array(1, bFreeMemory) );
 	}
 
 
 protected:
 
+	void *					Get_Record		(size_t i) const
+	{
+		return( m_Stack.Get_Entry((sLong)(i + 1)) );
+	}
+
 	void *					Get_Record_Push	(void)
 	{
-		if( m_Size < m_Buffer || _Grow() )
-		{
-			m_Size++;
-
-			return( (void *)(((char *)m_Stack) + m_RecordSize * (m_Size - 1)) );
-		}
-
-		return( NULL );
+		return( m_Stack.Inc_Array() ? m_Stack.Get_Entry(m_Stack.Get_Size() - 1) : NULL );
 	}
 
 	void *					Get_Record_Pop	(void)
 	{
-		if( m_Size > 0 )
+		if( m_Stack.Get_Size() > 0 )
 		{
-			m_Size--;
+			memcpy(m_Stack.Get_Array(), m_Stack.Get_Entry(m_Stack.Get_Size() - 1), m_Stack.Get_Value_Size());
 
-			return( (void *)(((char *)m_Stack) + m_RecordSize * (m_Size)) );
+			m_Stack.Dec_Array(true);
+
+			return( m_Stack.Get_Array() );
 		}
 
 		return( NULL );
@@ -971,25 +969,7 @@ protected:
 
 private:
 
-	size_t					m_Size, m_Buffer, m_RecordSize;
-
-	void					*m_Stack;
-
-
-	virtual bool			_Grow			(void)
-	{
-		void	*Stack	= SG_Realloc(m_Stack, (m_Buffer + 256) * m_RecordSize);
-
-		if( Stack )
-		{
-			m_Stack		= Stack;
-			m_Buffer	+= 256;
-
-			return( true );
-		}
-
-		return( false );
-	}
+	CSG_Array				m_Stack;
 
 };
 
@@ -1022,6 +1002,31 @@ typedef enum
 }
 TSG_Data_Type;
 
+//---------------------------------------------------------
+#define SG_DATATYPES_Undefined 0x0000
+#define SG_DATATYPES_Bit       0x0001
+#define SG_DATATYPES_Byte      0x0002
+#define SG_DATATYPES_Char      0x0004
+#define SG_DATATYPES_Word      0x0008
+#define SG_DATATYPES_Short     0x0010
+#define SG_DATATYPES_DWord     0x0020
+#define SG_DATATYPES_Int       0x0040
+#define SG_DATATYPES_ULong     0x0080
+#define SG_DATATYPES_Long      0x0100
+#define SG_DATATYPES_Float     0x0200
+#define SG_DATATYPES_Double    0x0400
+#define SG_DATATYPES_String    0x0800
+#define SG_DATATYPES_Date      0x1000
+#define SG_DATATYPES_Color     0x2000
+#define SG_DATATYPES_Binary    0x4000
+#define SG_DATATYPES_Standard  0xFFFF // all flags set
+#define SG_DATATYPES_SInteger  (SG_DATATYPES_Char|SG_DATATYPES_Short|SG_DATATYPES_Int  |SG_DATATYPES_Long)
+#define SG_DATATYPES_UInteger  (SG_DATATYPES_Byte|SG_DATATYPES_Word |SG_DATATYPES_DWord|SG_DATATYPES_ULong)
+#define SG_DATATYPES_Integer   (SG_DATATYPES_SInteger|SG_DATATYPES_UInteger)
+#define SG_DATATYPES_Real      (SG_DATATYPES_Float|SG_DATATYPES_Double)
+#define SG_DATATYPES_Numeric   (SG_DATATYPES_Integer|SG_DATATYPES_Real)
+#define SG_DATATYPES_Table     (SG_DATATYPES_String|SG_DATATYPES_Date|SG_DATATYPES_Color|SG_DATATYPES_Numeric|SG_DATATYPES_Binary)
+
 //---------------------------------------------------------
 const char	gSG_Data_Type_Identifier[][32]	=
 {
@@ -1071,6 +1076,7 @@ inline size_t	SG_Data_Type_Get_Size	(TSG_Data_Type Type)
 SAGA_API_DLL_EXPORT CSG_String		SG_Data_Type_Get_Name		(TSG_Data_Type Type, bool bShort = false);
 SAGA_API_DLL_EXPORT CSG_String		SG_Data_Type_Get_Identifier	(TSG_Data_Type Type);
 SAGA_API_DLL_EXPORT TSG_Data_Type	SG_Data_Type_Get_Type		(const CSG_String &Identifier);
+SAGA_API_DLL_EXPORT int				SG_Data_Type_Get_Flag		(TSG_Data_Type Type);
 SAGA_API_DLL_EXPORT bool			SG_Data_Type_is_Numeric		(TSG_Data_Type Type);
 SAGA_API_DLL_EXPORT bool			SG_Data_Type_Range_Check	(TSG_Data_Type Type, double &Value);
 
@@ -1210,7 +1216,7 @@ protected:
 
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT bool			SG_Dir_Exists				(const CSG_String &Directory);
-SAGA_API_DLL_EXPORT bool			SG_Dir_Create				(const CSG_String &Directory);
+SAGA_API_DLL_EXPORT bool			SG_Dir_Create				(const CSG_String &Directory, bool bFullPath = false);
 SAGA_API_DLL_EXPORT bool			SG_Dir_Delete				(const CSG_String &Directory, bool bRecursive = false);
 SAGA_API_DLL_EXPORT CSG_String		SG_Dir_Get_Current			(void);
 SAGA_API_DLL_EXPORT CSG_String		SG_Dir_Get_Temp				(void);
@@ -1607,6 +1613,17 @@ typedef int (* TSG_PFNC_UI_Callback) (TSG_UI_Callback_ID ID, CSG_UI_Parameter &P
 SAGA_API_DLL_EXPORT bool					SG_Set_UI_Callback			(TSG_PFNC_UI_Callback Function);
 SAGA_API_DLL_EXPORT TSG_PFNC_UI_Callback	SG_Get_UI_Callback			(void);
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Set_UTF8		(bool bOn);
+SAGA_API_DLL_EXPORT bool					SG_UI_Console_Get_UTF8		(void);
+
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Print_StdOut	(const char       *Text, SG_Char End = '\n', bool bFlush = true);
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Print_StdOut	(const wchar_t    *Text, SG_Char End = '\n', bool bFlush = true);
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Print_StdOut	(const CSG_String &Text, SG_Char End = '\n', bool bFlush = true);
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Print_StdErr	(const char       *Text, SG_Char End = '\n', bool bFlush = true);
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Print_StdErr	(const wchar_t    *Text, SG_Char End = '\n', bool bFlush = true);
+SAGA_API_DLL_EXPORT void					SG_UI_Console_Print_StdErr	(const CSG_String &Text, SG_Char End = '\n', bool bFlush = true);
+
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT int						SG_UI_Progress_Lock			(bool bOn);
 SAGA_API_DLL_EXPORT int						SG_UI_Progress_Reset		(void);
@@ -1629,9 +1646,16 @@ SAGA_API_DLL_EXPORT bool					SG_UI_Dlg_Parameters		(class CSG_Parameters *pParam
 
 SAGA_API_DLL_EXPORT int						SG_UI_Msg_Lock				(bool bOn);
 SAGA_API_DLL_EXPORT int						SG_UI_Msg_Reset				(void);
-SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add				(const CSG_String &Message, bool bNewLine, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add				(const char       *Message, bool bNewLine = true, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add				(const wchar_t    *Message, bool bNewLine = true, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add				(const CSG_String &Message, bool bNewLine = true, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Execution		(const char       *Message, bool bNewLine = true, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Execution		(const wchar_t    *Message, bool bNewLine = true, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Execution		(const CSG_String &Message, bool bNewLine = true, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Error			(const char       *Message);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Error			(const wchar_t    *Message);
 SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Error			(const CSG_String &Message);
-SAGA_API_DLL_EXPORT void					SG_UI_Msg_Add_Execution		(const CSG_String &Message, bool bNewLine, TSG_UI_MSG_STYLE Style = SG_UI_MSG_STYLE_NORMAL);
+SAGA_API_DLL_EXPORT void					SG_UI_Msg_Flush				(void);
 
 SAGA_API_DLL_EXPORT void					SG_UI_ProgressAndMsg_Lock	(bool bOn);
 SAGA_API_DLL_EXPORT void					SG_UI_ProgressAndMsg_Reset	(void);
