@@ -439,6 +439,27 @@ CSG_Parameter * CSG_Parameters::Add_Info_Range(const CSG_String &ParentID, const
 	return( _Add_Range(ParentID, ID, Name, Description,  true, Range_Min, Range_Max, 0.0, false, 0.0, false) );
 }
 
+//---------------------------------------------------------
+/**
+* Add a parameter for selecting a data type from a list of
+* choices. The Data_Types argument expects a combination
+* of data type flags as defined in <api_core.h>, e.g.
+* SG_DATATYPES_Integer or SG_DATATYPES_Numeric. You can add
+* an additional choice with the User argument, e.g. to offer
+* a type related to an selected input data set.
+*/
+//---------------------------------------------------------
+CSG_Parameter * CSG_Parameters::Add_Data_Type(const CSG_String &ParentID, const CSG_String &ID, const CSG_String &Name, const CSG_String &Description, int Data_Types, TSG_Data_Type Default, const CSG_String &User)
+{
+	CSG_Parameter *pParameter = _Add(ParentID, ID, Name, Description, PARAMETER_TYPE_Data_Type, 0);
+
+	bool bCallback = Set_Callback(false);
+	pParameter->asDataType()->Set_Data_Types(Data_Types, Default, User);
+	Set_Callback(bCallback);
+
+	return( pParameter );
+}
+
 //---------------------------------------------------------
 CSG_Parameter * CSG_Parameters::Add_Choice(const CSG_String &ParentID, const CSG_String &ID, const CSG_String &Name, const CSG_String &Description, const CSG_String &Items, int Default)
 {
@@ -446,7 +467,7 @@ CSG_Parameter * CSG_Parameters::Add_Choice(const CSG_String &ParentID, const CSG
 
 	pParameter->asChoice()->Set_Items(Items);
 
-	bool	bCallback	= Set_Callback(false);
+	bool bCallback = Set_Callback(false);
 	pParameter->Set_Value  (Default);
 	pParameter->Set_Default(Default);
 	Set_Callback(bCallback);
@@ -1052,6 +1073,7 @@ CSG_Parameter * CSG_Parameters::_Add(const CSG_String &ParentID, const CSG_Strin
 	case PARAMETER_TYPE_Degree           : pParameter = new CSG_Parameter_Degree            (this, Get_Parameter(ParentID), ID, Name, Description, Constraint); break;
 	case PARAMETER_TYPE_Date             : pParameter = new CSG_Parameter_Date              (this, Get_Parameter(ParentID), ID, Name, Description, Constraint); break;
 	case PARAMETER_TYPE_Range            : pParameter = new CSG_Parameter_Range             (this, Get_Parameter(ParentID), ID, Name, Description, Constraint); break;
+	case PARAMETER_TYPE_Data_Type        : pParameter = new CSG_Parameter_Data_Type         (this, Get_Parameter(ParentID), ID, Name, Description, Constraint); break;
 	case PARAMETER_TYPE_Choice           : pParameter = new CSG_Parameter_Choice            (this, Get_Parameter(ParentID), ID, Name, Description, Constraint); break;
 	case PARAMETER_TYPE_Choices          : pParameter = new CSG_Parameter_Choices           (this, Get_Parameter(ParentID), ID, Name, Description, Constraint); break;
 
@@ -1518,119 +1540,134 @@ bool CSG_Parameters::DataObjects_Create(void)
 {
 	bool bResult = true;
 
-	for(int i=0; i<Get_Count() && bResult; i++)
+	for(int i=0; bResult && i<Get_Count(); i++)
 	{
-		CSG_Parameter *p = m_Parameters[i];
+		CSG_Parameter &P = *m_Parameters[i];
 
 		//-------------------------------------------------
-		if( p->Get_Type() == PARAMETER_TYPE_Parameters )
+		if( P.Get_Type() == PARAMETER_TYPE_Parameters )
 		{
-			bResult = p->asParameters()->DataObjects_Create();
-		}
-		else if( p->Get_Type() == PARAMETER_TYPE_DataObject_Output )
-		{
-			if( m_pManager || p->asDataObject() == DATAOBJECT_CREATE )
+			if( !P.asParameters()->DataObjects_Create() )
 			{
-				p->Set_Value(DATAOBJECT_NOTSET);
+				bResult = false;
 			}
 		}
-		else if( p->is_Input() )
+		else if( P.Get_Type() == PARAMETER_TYPE_DataObject_Output )
 		{
-			bResult = !p->is_Enabled() || p->Check(true);
+			if( m_pManager || P.asDataObject() == DATAOBJECT_CREATE )
+			{
+				P.Set_Value(DATAOBJECT_NOTSET);
+			}
+		}
+		else if( P.is_Input() )
+		{
+			if( P.is_Enabled() && P.Check(true) == false )
+			{
+				bResult = false;
+
+				SG_UI_Dlg_Message(CSG_String::Format("%s\n[%s]\n%s", _TL("Input Error"), P.Get_Identifier(), P.Get_Name()), Get_Name());
+			}
 		}
 
 		//-------------------------------------------------
-		else if( p->is_DataObject_List() )
+		else if( P.is_DataObject_List() )
 		{
-			for(int j=p->asList()->Get_Item_Count()-1; j>=0; j--)
+			for(int j=P.asList()->Get_Item_Count()-1; j>=0; j--)
 			{
-				if( m_pManager && !m_pManager->Exists(p->asList()->Get_Item(j)) )
+				if( m_pManager && !m_pManager->Exists(P.asList()->Get_Item(j)) )
 				{
-					p->asList()->Del_Item(j);
+					P.asList()->Del_Item(j);
 				}
 			}
 		}
 
 		//-------------------------------------------------
-		else if( p->is_DataObject() && p->is_Enabled() == false )
+		else if( P.is_DataObject() && P.is_Enabled() == false )
 		{
-			if( p->asDataObject() != DATAOBJECT_CREATE && (!m_pManager || !m_pManager->Exists(p->asDataObject())) )
+			if( P.asDataObject() != DATAOBJECT_CREATE && (!m_pManager || !m_pManager->Exists(P.asDataObject())) )
 			{
-				p->Set_Value(DATAOBJECT_NOTSET);
+				P.Set_Value(DATAOBJECT_NOTSET);
 			}
 		}
 
-		else if( p->is_DataObject() )
+		else if( P.is_DataObject() )
 		{
-			CSG_Data_Object *pDataObject = p->asDataObject();
+			CSG_Data_Object *pObject = P.asDataObject();
 
-			if( (pDataObject == DATAOBJECT_CREATE)
-			||  (pDataObject == DATAOBJECT_NOTSET && !p->is_Optional())
-			||  (pDataObject != DATAOBJECT_NOTSET && m_pManager && !m_pManager->Exists(pDataObject)) )
+			if( (pObject == DATAOBJECT_CREATE)
+			||  (pObject == DATAOBJECT_NOTSET && !P.is_Optional())
+			||  (pObject != DATAOBJECT_NOTSET && m_pManager && !m_pManager->Exists(pObject)) )
 			{
-				pDataObject = NULL;
-
-				switch( p->Get_Type() )
+				switch( P.Get_Type() )
 				{
-				case PARAMETER_TYPE_Table     : pDataObject = SG_Create_Table     (); break;
-				case PARAMETER_TYPE_TIN       : pDataObject = SG_Create_TIN       (); break;
-				case PARAMETER_TYPE_PointCloud: pDataObject = SG_Create_PointCloud(); break;
-				case PARAMETER_TYPE_Shapes    : pDataObject = SG_Create_Shapes(
-						((CSG_Parameter_Shapes *)p)->Get_Shape_Type()
-					);
-					break;
+				case PARAMETER_TYPE_Table     : pObject = SG_Create_Table     (); break;
+				case PARAMETER_TYPE_Shapes    : pObject = SG_Create_Shapes    (); break;
+				case PARAMETER_TYPE_PointCloud: pObject = SG_Create_PointCloud(); break;
+				case PARAMETER_TYPE_TIN       : pObject = SG_Create_TIN       (); break;
+				case PARAMETER_TYPE_Grid      : pObject = SG_Create_Grid      (); break;
+				case PARAMETER_TYPE_Grids     : pObject = SG_Create_Grids     (); break;
+				default                       : pObject = NULL                  ; break;
+				}
+			}
 
-				case PARAMETER_TYPE_Grid      :
-				case PARAMETER_TYPE_Grids     :
-					if(	p->Get_Parent() && p->Get_Parent()->Get_Type() == PARAMETER_TYPE_Grid_System 
-					&&	p->Get_Parent()->asGrid_System() && p->Get_Parent()->asGrid_System()->is_Valid() )
+			if( pObject )
+			{
+				if( P.Get_Type() == PARAMETER_TYPE_Shapes
+				&&  ((CSG_Parameter_Shapes *)&P)->Get_Shape_Type() != SHAPE_TYPE_Undefined
+				&&  ((CSG_Parameter_Shapes *)&P)->Get_Shape_Type() != pObject->asShapes()->Get_Type() )
+				{
+					if( has_GUI() && pObject->asShapes()->Get_Type() != SHAPE_TYPE_Undefined )
 					{
-						if( p->Get_Type() == PARAMETER_TYPE_Grid )
+						pObject	= SG_Create_Shapes (((CSG_Parameter_Shapes *)&P)->Get_Shape_Type());
+					}
+					else
+					{
+						pObject->asShapes()->Create(((CSG_Parameter_Shapes *)&P)->Get_Shape_Type());
+					}
+				}
+
+				if( (P.Get_Type() == PARAMETER_TYPE_Grid || P.Get_Type() == PARAMETER_TYPE_Grids) )
+				{
+					if(	P.Get_Parent() && P.Get_Parent()->asGrid_System() && P.Get_Parent()->asGrid_System()->is_Valid() )
+					{
+						if( P.Get_Type() == PARAMETER_TYPE_Grid )
 						{
-							pDataObject = SG_Create_Grid(*p->Get_Parent()->asGrid_System(), ((CSG_Parameter_Grid *)p)->Get_Preferred_Type());
+							pObject->asGrid ()->Create(*P.Get_Parent()->asGrid_System()       , ((CSG_Parameter_Grid  *)&P)->Get_Preferred_Type());
 						}
 						else
 						{
-							pDataObject = SG_Create_Grids(*p->Get_Parent()->asGrid_System(), 0, 0.0, ((CSG_Parameter_Grids *)p)->Get_Preferred_Type());
+							pObject->asGrids()->Create(*P.Get_Parent()->asGrid_System(), 0, 0., ((CSG_Parameter_Grids *)&P)->Get_Preferred_Type());
 						}
 					}
-					break;
-
-				default:
-					break;
 				}
-			}
-			else if( p->Get_Type() == PARAMETER_TYPE_Shapes && p->asShapes() )
-			{
-				if( ((CSG_Parameter_Shapes *)p)->Get_Shape_Type() != SHAPE_TYPE_Undefined
-				&&	((CSG_Parameter_Shapes *)p)->Get_Shape_Type() != p->asShapes()->Get_Type() )
-				{
-					pDataObject	= SG_Create_Shapes(((CSG_Parameter_Shapes *)p)->Get_Shape_Type());
-				}
-			}
 
-			if( pDataObject )
-			{
-				if( p->Set_Value(pDataObject) )
+				if( P.Set_Value(pObject) )
 				{
-					pDataObject->Set_Name(p->Get_Name());
-					pDataObject->Get_MetaData().Del_Children();
+					pObject->Set_Name(P.Get_Name());
+					pObject->Get_MetaData().Del_Children();
 
 					if( m_pManager )
 					{
-						m_pManager->Add(pDataObject);
+						m_pManager->Add(pObject);
 					}
 				}
 				else
 				{
-					delete(pDataObject);
+					delete(pObject);
 
 					bResult = false;
+
+					SG_UI_Dlg_Message(CSG_String::Format("%s\n[%s]\n%s", _TL("Output Error"), P.Get_Identifier(), P.Get_Name()), Get_Name());
 				}
 			}
-			else
+			else // if( pObject == NULL )
 			{
-				bResult = p->is_Optional();
+				if( !P.is_Optional() )
+				{
+					bResult = false;
+
+					SG_UI_Dlg_Message(CSG_String::Format("%s\n[%s]\n%s", _TL("Output Error"), P.Get_Identifier(), P.Get_Name()), Get_Name());
+				}
 			}
 		}
 	}
