@@ -318,6 +318,7 @@ bool CSG_Tool_Chain::Create(const CSG_MetaData &Chain)
 		case PARAMETER_TYPE_Date             : Parameters.Add_Date           (ParentID, ID, Name, Desc, 0.)->Set_Value(Value);	break;
 
 		case PARAMETER_TYPE_Range            : Parameters.Add_Range          (ParentID, ID, Name, Desc, Value.BeforeFirst(';').asDouble(), Value.AfterFirst(';').asDouble(), Min, bMin, Max, bMax);	break;
+		case PARAMETER_TYPE_Data_Type        :
 		case PARAMETER_TYPE_Choice           : Parameters.Add_Choice         (ParentID, ID, Name, Desc, Parameter.Get_Content("choices"))->Set_Value(Value);	break;
 		case PARAMETER_TYPE_Choices          : Parameters.Add_Choices        (ParentID, ID, Name, Desc, Parameter.Get_Content("choices"))->Set_Value(Value);	break;
 
@@ -325,9 +326,9 @@ bool CSG_Tool_Chain::Create(const CSG_MetaData &Chain)
 		case PARAMETER_TYPE_Text             : Parameters.Add_String         (ParentID, ID, Name, Desc, Value,  true);	break;
 
 		case PARAMETER_TYPE_FilePath         : Parameters.Add_FilePath       (ParentID, ID, Name, Desc, Parameter.Get_Content("filter"), Value,
-													IS_TRUE_PROPERTY(Parameter, "save"     ),
-													IS_TRUE_PROPERTY(Parameter, "directory"),
-													IS_TRUE_PROPERTY(Parameter, "multiple" )
+												IS_TRUE_PROPERTY(Parameter, "save"     ),
+												IS_TRUE_PROPERTY(Parameter, "directory"),
+												IS_TRUE_PROPERTY(Parameter, "multiple" )
 											); break;
 
 		case PARAMETER_TYPE_Font             : break;
@@ -677,9 +678,12 @@ bool CSG_Tool_Chain::Data_Finalize(void)
 	{
 		if( Parameters(i)->is_DataObject() )
 		{
-			if( Parameters(i)->Get_Type() == PARAMETER_TYPE_DataObject_Output && m_Data(Parameters(i)->Get_Identifier()) )
+			if( m_Data(Parameters(i)->Get_Identifier()) )
 			{
-				Parameters(i)->Set_Value(m_Data(Parameters(i)->Get_Identifier())->asDataObject());
+				if( Parameters(i)->Get_Type() == PARAMETER_TYPE_DataObject_Output || (Parameters(i)->is_Output() && !Parameters(i)->is_Optional()) )
+				{
+					Parameters(i)->Set_Value(m_Data(Parameters(i)->Get_Identifier())->asDataObject());
+				}
 			}
 
 			m_Data_Manager.Delete(Parameters(i)->asDataObject(), true);
@@ -838,7 +842,7 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 	switch( pOption->Get_Type() )
 	{
 	//-----------------------------------------------------
-	case PARAMETER_TYPE_Bool    :
+	case PARAMETER_TYPE_Bool     :
 		{
 			CSG_String	Value;
 
@@ -851,9 +855,10 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 		break;
 
 	//-----------------------------------------------------
-	case PARAMETER_TYPE_Int     :
-	case PARAMETER_TYPE_Color   :
-	case PARAMETER_TYPE_Choice  :
+	case PARAMETER_TYPE_Int      :
+	case PARAMETER_TYPE_Color    :
+	case PARAMETER_TYPE_Data_Type:
+	case PARAMETER_TYPE_Choice   :
 		{
 			int		Value;
 
@@ -868,8 +873,8 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 		break;
 
 	//-----------------------------------------------------
-	case PARAMETER_TYPE_Double  :
-	case PARAMETER_TYPE_Degree  :
+	case PARAMETER_TYPE_Double   :
+	case PARAMETER_TYPE_Degree   :
 		{
 			double	Value;
 
@@ -884,9 +889,9 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 		break;
 
 	//-----------------------------------------------------
-	case PARAMETER_TYPE_String  :
-	case PARAMETER_TYPE_Text    :
-	case PARAMETER_TYPE_FilePath:
+	case PARAMETER_TYPE_String   :
+	case PARAMETER_TYPE_Text     :
+	case PARAMETER_TYPE_FilePath :
 		{
 			CSG_String	Value;
 
@@ -901,6 +906,21 @@ bool CSG_Tool_Chain::Check_Condition(const CSG_MetaData &Condition, CSG_Paramete
 		}
 		break;
 
+	//-----------------------------------------------------
+	case PARAMETER_TYPE_Grid_System:
+		{
+			CSG_String	Value;
+
+			if( Condition.Get_Property("value", Value) )
+			{
+				bool bValid = pOption->asGrid_System()->is_Valid();
+
+				if(      !Type.CmpNoCase("=") || !Type.CmpNoCase("equal"    ) )	{	return( (IS_TRUE_STRING(Value) ?  bValid : !bValid) );	}
+				else if( !Type.CmpNoCase("!") || !Type.CmpNoCase("not_equal") )	{	return( (IS_TRUE_STRING(Value) ? !bValid :  bValid) );	}
+			}
+		}
+		break;
+
 	//-----------------------------------------------------
 	default:
 		// nop
@@ -1245,7 +1265,7 @@ bool CSG_Tool_Chain::Tool_Run(const CSG_MetaData &Tool, bool bShowError)
 
 	pTool->Settings_Push(&m_Data_Manager);
 
-	bool	bResult	= false;
+	bool bResult = false;
 
 	if( !pTool->On_Before_Execution() )
 	{
@@ -1257,7 +1277,7 @@ bool CSG_Tool_Chain::Tool_Run(const CSG_MetaData &Tool, bool bShowError)
 	}
 	else if( !(bResult = pTool->Execute(m_bAddHistory)) )
 	{
-	//	if( bShowError ) Error_Fmt("%s [%s].[%s]", _TL("tool execution failed"             ), pTool->Get_Library().c_str(), pTool->Get_Name().c_str());
+		Message_Fmt               ("%s [%s].[%s]", _TL("tool execution failed"             ), pTool->Get_Library().c_str(), pTool->Get_Name().c_str());
 	}
 
 	if( bResult )
@@ -1378,9 +1398,9 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// check for invalid parameters...
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( Parameter.Cmp_Name("comment") ) { continue; }
 
-		CSG_Parameter	*pParameter, *pOwner;
+		CSG_Parameter *pParameter, *pOwner;
 
 		if( !Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner) )
 		{
@@ -1390,146 +1410,166 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 		}
 	}
 
+	//-----------------------------------------------------
+	if( Tool.Get_Property("grid_system") )
+	{
+		CSG_Parameter *pParameter = Parameters(Tool.Get_Property("grid_system"));
+
+		if( pParameter && pParameter->asGrid_System() )
+		{
+			if( !pTool->Set_Grid_System(*pParameter->asGrid_System()) )
+			{
+				Message_Fmt("\n%s: %s\n", _TL("Warning"), _TL("failed to set tool's grid system"));
+			}
+		}
+	}
+
+	//-----------------------------------------------------
+	for(int i=0; i<Tool.Get_Children_Count(); i++)	// initialize all options first, some might en-/disable mandatory input data
+	{
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("option") ) { continue; }
+
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+
+		if( IS_TRUE_PROPERTY(Parameter, "varname")
+			? pParameter->Set_Value(Parameters(Parameter.Get_Content()))
+			: pParameter->Set_Value(           Parameter.Get_Content() ) )
+		{
+			pParameter->has_Changed(); if( pOwner ) { pOwner->has_Changed(); }
+		}
+	}
+
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// set data input first
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("input") ) { continue; }
 
-		if( Parameter.Cmp_Name("input") )
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+
+		int	Index;
+
+		if( Parameter.Get_Content().Find('[') < 1 || !Parameter.Get_Content().AfterFirst('[').asInt(Index) )
 		{
-			CSG_Parameter	*pParameter, *pOwner;	Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+			Index = -1;
+		}
 
-			int	Index;
+		bool bResult = false;
 
-			if( Parameter.Get_Content().Find('[') < 1 || !Parameter.Get_Content().AfterFirst('[').asInt(Index) )
+		CSG_Parameter *pData = m_Data(Index < 0 ? Parameter.Get_Content() : Parameter.Get_Content().BeforeFirst('['));
+
+		if( pData && (pParameter->is_DataObject() || pParameter->is_DataObject_List()) )
+		{
+			if( Index >= 0 && (pData->is_DataObject_List() || pData->asGrids()) )
 			{
-				Index	= -1;
-			}
+				CSG_Data_Object	*pObject = pData->is_DataObject_List()
+					? pData->asList ()->Get_Item    (Index)
+					: pData->asGrids()->Get_Grid_Ptr(Index);
 
-			bool	bResult	= false;
-
-			CSG_Parameter	*pData	= m_Data(Index < 0 ? Parameter.Get_Content() : Parameter.Get_Content().BeforeFirst('['));
-
-			if( pData && (pParameter->is_DataObject() || pParameter->is_DataObject_List()) )
-			{
-				if( Index >= 0 && (pData->is_DataObject_List() || pData->asGrids()) )
+				if( pParameter->is_DataObject() )
 				{
-					CSG_Data_Object	*pObject	= pData->is_DataObject_List()
-						? pData->asList ()->Get_Item    (Index)
-						: pData->asGrids()->Get_Grid_Ptr(Index);
-
-					if( pParameter->is_DataObject() )
-					{
-						bResult	= pParameter->Set_Value(pObject);
-					}
-					if( pParameter->is_DataObject_List() )
-					{
-						bResult	= pParameter->asList()->Add_Item(pObject);
-					}
+					bResult = pParameter->Set_Value(pObject);
 				}
-				else if( pParameter->Get_Type() == pData->Get_Type() )
+				if( pParameter->is_DataObject_List() )
 				{
-					if( pParameter->is_DataObject_List() && pParameter->asList()->Get_Item_Count() > 0 )
-					{
-						bResult	= true;
-
-						for(int i=0; bResult && i<pData->asList()->Get_Item_Count(); i++)
-						{
-							bResult	= pParameter->asList()->Add_Item(pData->asList()->Get_Item(i));
-						}
-					}
-					else
-					{
-						bResult	= pParameter->Assign(pData);
-					}
-				}
-				else if( pParameter->is_DataObject_List() && pData->is_DataObject() )
-				{
-					bResult	= pParameter->asList()->Add_Item(pData->asDataObject());
-				}
-				else if( Parameter_isCompatible(pParameter->Get_Type(), pData->Get_Type()) )
-				{
-					bResult = pParameter->Set_Value(pData->asDataObject());
+					bResult = pParameter->asList()->Add_Item(pObject);
 				}
 			}
-
-			if( !bResult )
+			else if( pParameter->Get_Type() == pData->Get_Type() )
 			{
-				Error_Fmt("%s: %s", _TL("set input"), Parameter.Get_Property("id"));
+				if( pParameter->is_DataObject_List() && pParameter->asList()->Get_Item_Count() > 0 )
+				{
+					bResult = true;
 
-				return( false );
+					for(int i=0; bResult && i<pData->asList()->Get_Item_Count(); i++)
+					{
+						bResult = pParameter->asList()->Add_Item(pData->asList()->Get_Item(i));
+					}
+				}
+				else
+				{
+					bResult = pParameter->Assign(pData);
+				}
 			}
-
-			pParameter->has_Changed();
-
-			if( pOwner )
+			else if( pParameter->is_DataObject_List() && pData->is_DataObject() )
 			{
-				pOwner->has_Changed();
+				bResult = pParameter->asList()->Add_Item(pData->asDataObject());
 			}
+			else if( Parameter_isCompatible(pParameter->Get_Type(), pData->Get_Type()) )
+			{
+				bResult = pParameter->Set_Value(pData->asDataObject());
+			}
+		}
+
+		if( bResult )
+		{
+			pParameter->has_Changed(); if( pOwner ) { pOwner->has_Changed(); }
+		}
+		else if( pParameter->is_Enabled() )
+		{
+			Error_Fmt("%s: %s", _TL("set input"), Parameter.Get_Property("id"));
+
+			return( false );
 		}
 	}
 
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// now set all options
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("option") ) { continue; }
 
-		if( Parameter.Cmp_Name("option") )
-		{
-			CSG_Parameter	*pParameter, *pOwner;	Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
 
-			if( IS_TRUE_PROPERTY(Parameter, "varname") )
-			{	// does option want a value from tool chain parameters and do these provide one ?
-				switch( pParameter->Get_Type() )
-				{
-				default:
-					pParameter->Set_Value(Parameters(Parameter.Get_Content()));
-					break;
-
-				case PARAMETER_TYPE_FixedTable:
-					if( m_Data(Parameter.Get_Content()) && pParameter->asTable()->Assign_Values(m_Data(Parameter.Get_Content())->asTable()) )
-					{
-						pParameter->has_Changed();
-					}
-					break;
-				}
-			}
-			else
+		if( IS_TRUE_PROPERTY(Parameter, "varname") )
+		{	// does option want a value from tool chain parameters and do these provide one ?
+			switch( pParameter->Get_Type() )
 			{
-				switch( pParameter->Get_Type() )
+			default:
+				pParameter->Set_Value(Parameters(Parameter.Get_Content()));
+				break;
+
+			case PARAMETER_TYPE_FixedTable:
+				if( m_Data(Parameter.Get_Content()) && pParameter->asTable()->Assign_Values(m_Data(Parameter.Get_Content())->asTable()) )
 				{
-				case PARAMETER_TYPE_FixedTable:
-					if( Parameter("OPTION") )
-					{
-						pParameter->Serialize(*Parameter("OPTION"), false);
-					}
-					break;
+					pParameter->has_Changed();
+				}
+				break;
+			}
+		}
+		else
+		{
+			switch( pParameter->Get_Type() )
+			{
+			case PARAMETER_TYPE_FixedTable:
+				if( Parameter("OPTION") )
+				{
+					pParameter->Serialize(*Parameter("OPTION"), false);
+				}
+				break;
 
-				default: {
-					CSG_String	Value(Parameter.Get_Content());
+			default: {
+				CSG_String	Value(Parameter.Get_Content());
 
-					if( Value.Find("$(") >= 0 )
+				if( Value.Find("$(") >= 0 )
+				{
+					for(int j=0; j<Parameters.Get_Count(); j++)
 					{
-						for(int j=0; j<Parameters.Get_Count(); j++)
+						CSG_String	Var; Var.Printf("$(%s)", Parameters(j)->Get_Identifier());
+
+						if( Value.Find(Var) >= 0 )
 						{
-							CSG_String	Var; Var.Printf("$(%s)", Parameters(j)->Get_Identifier());
-
-							if( Value.Find(Var) >= 0 )
-							{
-								Value.Replace(Var, Parameters(j)->asString());
-							}
+							Value.Replace(Var, Parameters(j)->asString());
 						}
 					}
-
-					pParameter->Set_Value(Value);
-
-					if( pOwner )
-					{
-						pOwner->has_Changed();
-					}
-
-					break; }
 				}
+
+				pParameter->Set_Value(Value);
+
+				if( pOwner )
+				{
+					pOwner->has_Changed();
+				}
+
+				break; }
 			}
 		}
 	}
@@ -1537,26 +1577,23 @@ bool CSG_Tool_Chain::Tool_Initialize(const CSG_MetaData &Tool, CSG_Tool *pTool)
 	//-----------------------------------------------------
 	for(int i=0; i<Tool.Get_Children_Count(); i++)	// finally set the data output
 	{
-		const CSG_MetaData	&Parameter	= Tool[i];	if( Parameter.Cmp_Name("comment") )	{	continue;	}
+		const CSG_MetaData &Parameter = Tool[i]; if( !Parameter.Cmp_Name("output") ) { continue; }
 
-		if( Parameter.Cmp_Name("output") )
+		CSG_Parameter *pParameter, *pOwner; Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
+
+		if( !pParameter->Assign(m_Data(Parameter.Get_Content())) )
 		{
-			CSG_Parameter	*pParameter, *pOwner;	Tool_Get_Parameter(Parameter, pTool, &pParameter, &pOwner);
-
-			if( !pParameter->Assign(m_Data(Parameter.Get_Content())) )
+			if( pParameter->is_DataObject() )
 			{
-				if( pParameter->is_DataObject() )
-				{
-					pParameter->Set_Value(DATAOBJECT_CREATE);
-				}
-				else if( pParameter->is_DataObject_List() )
-				{
-					pParameter->asList()->Del_Items();
-				}
-				else if( pParameter->asGrids() )
-				{
-					pParameter->asGrids()->Del_Grids();
-				}
+				pParameter->Set_Value(DATAOBJECT_CREATE);
+			}
+			else if( pParameter->is_DataObject_List() )
+			{
+				pParameter->asList()->Del_Items();
+			}
+			else if( pParameter->asGrids() )
+			{
+				pParameter->asGrids()->Del_Grids();
 			}
 		}
 	}
@@ -1719,6 +1756,7 @@ bool CSG_Tool_Chain::_Get_Script_Tool(CSG_MetaData &Tool, CSG_Parameters *pParam
 			pChild	= Tool.Add_Child("option", pParameter->asString());
 			break;
 
+		case PARAMETER_TYPE_Data_Type   :
 		case PARAMETER_TYPE_Choice      :
 			pChild	= Tool.Add_Child("option", pParameter->asInt());
 			break;
@@ -1904,10 +1942,10 @@ CSG_Tool_Chains::CSG_Tool_Chains(const CSG_String &Library_Name, const CSG_Strin
 	//-----------------------------------------------------
 	if( m_Library_Name.is_Empty() || !m_Library_Name.Cmp("toolchains") )
 	{
-		m_Library_Name	= "toolchains";
-		m_Name			= _TL("Tool Chains");
-		m_Description	= _TL("Unsorted tool chains");
-		m_Menu			= _TL("Tool Chains");
+		m_Library_Name	= "_tool_chains_uncategorized";
+		m_Name			= _TL("Uncategorized Tool Chains");
+		m_Description	= _TL("Uncategorized Tool Chains");
+		m_Menu			= _TL("Uncategorized Tool Chains");
 	}
 
 	//-----------------------------------------------------
@@ -2163,6 +2201,7 @@ bool CSG_Tool_Chain::_Save_History_Add_Tool(const CSG_MetaData &History, CSG_Met
 				pParameter	= Tool.Add_Child("option", pChild->Get_Content());
 				break;
 
+			case PARAMETER_TYPE_Data_Type   :
 			case PARAMETER_TYPE_Choice      :
 				pParameter	= Tool.Add_Child("option", pChild->Get_Property("index"));
 				break;
