@@ -233,7 +233,7 @@ bool		Execute(int argc, char *argv[])
 	//-----------------------------------------------------
 	if( argc == 3 && CMD_Get_XML() )
 	{	// Just output tool synopsis as XML-tagged text, then return.
-		SG_Printf(pTool->Get_Summary(true, "", "", SG_SUMMARY_FMT_XML).c_str());
+		CMD_Print(pTool->Get_Summary(true, "", "", SG_SUMMARY_FMT_XML));
 
 		return( true );
 	}
@@ -526,8 +526,8 @@ bool		Check_Flags		(const CSG_String &Argument)
 
 		if( s.Find('l') >= 0 )	// l: load translation dictionary
 		{
-			SG_Printf(CSG_String::Format("\n%s:", _TL("loading translation dictionary")));
-			SG_Printf(CSG_String::Format("\n%s.\n",
+			CMD_Print(CSG_String::Format("\n%s:", _TL("loading translation dictionary")));
+			CMD_Print(CSG_String::Format("\n%s.\n",
 				SG_Get_Translator().Create(SG_File_Make_Path(Path_Shared, SG_T("saga"), SG_T("lng")), false)
 				? _TL("success") : _TL("failed")
 			));
@@ -575,7 +575,7 @@ bool		Check_Flags		(const CSG_String &Argument)
 	{
 		int	On; if( !CSG_String(Argument).AfterFirst('=').asInt(On) ) { On = 1; } // let's switch it on by default!
 
-		CMD_Set_UTF8(On);
+		SG_UI_Console_Set_UTF8(On);
 
 		return( true );
 	}
@@ -598,7 +598,7 @@ void		Print_Libraries	(void)
 	{
 		if( CMD_Get_XML() )
 		{
-			SG_Printf(SG_Get_Tool_Library_Manager().Get_Summary(SG_SUMMARY_FMT_XML).c_str());
+			CMD_Print(SG_Get_Tool_Library_Manager().Get_Summary(SG_SUMMARY_FMT_XML));
 		}
 		else
 		{
@@ -620,11 +620,11 @@ void		Print_Tools		(const CSG_String &Library)
 		{
 			for(int i=0; i<SG_Get_Tool_Library_Manager().Get_Count(); i++)
 			{
-				CSG_Tool_Library	*pLibrary	= SG_Get_Tool_Library_Manager().Get_Library(i);
+				CSG_Tool_Library *pLibrary = SG_Get_Tool_Library_Manager().Get_Library(i);
 
 				if( !pLibrary->Get_Library_Name().Cmp(Library) )
 				{
-					SG_Printf(pLibrary->Get_Summary(SG_SUMMARY_FMT_XML , false).c_str());
+					CMD_Print(pLibrary->Get_Summary(SG_SUMMARY_FMT_XML , false));
 				}
 			}
 		}
@@ -632,7 +632,7 @@ void		Print_Tools		(const CSG_String &Library)
 		{
 			for(int i=0; i<SG_Get_Tool_Library_Manager().Get_Count(); i++)
 			{
-				CSG_Tool_Library	*pLibrary	= SG_Get_Tool_Library_Manager().Get_Library(i);
+				CSG_Tool_Library *pLibrary = SG_Get_Tool_Library_Manager().Get_Library(i);
 
 				if( !pLibrary->Get_Library_Name().Cmp(Library) )
 				{
@@ -654,29 +654,26 @@ void		Print_Execution	(CSG_Tool *pTool)
 	{
 		if( CMD_Get_XML() )
 		{
-			#define SG_XML_LIBRARY		SG_T("library")
-			#define SG_XML_LIBRARY_PATH	SG_T("path")
-			#define SG_XML_LIBRARY_NAME	SG_T("name")
-
-			SG_Printf("<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\n");
-			SG_Printf("<%s>\n", SG_XML_LIBRARY);
-			SG_Printf("\t<%s>%s</%s>\n", SG_XML_LIBRARY_PATH, pTool->Get_File_Name().c_str(), SG_XML_LIBRARY_PATH);
-			SG_Printf("\t<%s>%s</%s>\n", SG_XML_LIBRARY_NAME, pTool->Get_Library  ().c_str(), SG_XML_LIBRARY_NAME);
-			SG_Printf("</%s>\n", SG_XML_LIBRARY);
+			CMD_Print("<?xml version='1.0' encoding='UTF-8' standalone='yes' ?>\n");
+			CMD_Print("<library>");
+			CMD_Print(pTool->Get_File_Name(), "path");
+			CMD_Print(pTool->Get_Library  (), "name");
+			CMD_Print("</library>");
 		}
 		else
 		{
-			SG_Printf("____________________________\n");
-			SG_Printf("%s: %s\n", _TL("library path"), SG_File_Get_Path(pTool->Get_File_Name()       ).c_str());
-			SG_Printf("%s: %s\n", _TL("library name"), SG_File_Get_Name(pTool->Get_File_Name(), false).c_str());
-			SG_Printf("%s: %s\n", _TL("library     "), pTool->Get_Library().c_str());
-			SG_Printf("%s: %s\n", _TL("tool        "), pTool->Get_Name   ().c_str());
-			SG_Printf("%s: %s\n", _TL("identifier  "), pTool->Get_ID     ().c_str());
-			SG_Printf("%s: %s\n", _TL("author      "), pTool->Get_Author ().c_str());
+			CMD_Print("____________________________");
+			CMD_Print(CSG_String::Format("%s: %s", _TL("library path"), SG_File_Get_Path(pTool->Get_File_Name()       ).c_str()));
+			CMD_Print(CSG_String::Format("%s: %s", _TL("library path"), SG_File_Get_Path(pTool->Get_File_Name()       ).c_str()));
+			CMD_Print(CSG_String::Format("%s: %s", _TL("library name"), SG_File_Get_Name(pTool->Get_File_Name(), false).c_str()));
+			CMD_Print(CSG_String::Format("%s: %s", _TL("library     "), pTool->Get_Library().c_str()));
+			CMD_Print(CSG_String::Format("%s: %s", _TL("tool        "), pTool->Get_Name   ().c_str()));
+			CMD_Print(CSG_String::Format("%s: %s", _TL("identifier  "), pTool->Get_ID     ().c_str()));
+			CMD_Print(CSG_String::Format("%s: %s", _TL("author      "), pTool->Get_Author ().c_str()));
 		#ifdef _OPENMP
-			SG_Printf("%s: %d [%d]\n", _TL("processors  "), SG_OMP_Get_Max_Num_Threads(), SG_OMP_Get_Max_Num_Procs());
+			CMD_Print(CSG_String::Format("%s: %d [%d]", _TL("processors  "), SG_OMP_Get_Max_Num_Threads(), SG_OMP_Get_Max_Num_Procs()));
 		#endif // _OPENMP
-			SG_Printf("____________________________\n\n");
+			CMD_Print("____________________________\n");
 		}
 	}
 }
@@ -725,8 +722,7 @@ void		Print_Get_Help	(void)
 {
 	if( CMD_Get_Show_Messages() )
 	{
-		CMD_Print(_TL("type -h or --help for further information"));
-		CMD_Print("");
+		CMD_Print(CSG_String::Format("\n%s\n", _TL("type -h or --help for further information")));
 	}
 }
 
@@ -836,7 +832,7 @@ void		Print_Help		(const CSG_String &Library)
 {
 	Print_Logo();
 
-	CSG_Tool_Library	*pLibrary;
+	CSG_Tool_Library *pLibrary;
 
 	if( !Load_Libraries() || !(pLibrary = SG_Get_Tool_Library_Manager().Get_Library(Library, true)) )
 	{
@@ -860,7 +856,7 @@ void		Print_Help		(const CSG_String &Library, const CSG_String &Tool)
 	}
 	else
 	{
-		CSG_Tool	*pTool	= SG_Get_Tool_Library_Manager().Get_Tool(Library, Tool);
+		CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool(Library, Tool);
 
 		if( !pTool )
 		{
